﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using JoeScan.JCamNet;

namespace Examples
{
    public partial class Form1 : Form
    {
        private Thread scanThread;
        private volatile bool isRunning;
        private Scanner scanner; 
        
        public Form1()
        {
            InitializeComponent();
            scanner = SetupScanner();
        }

        // ScanThreadMain contains the code that is executed in a thread, i.e. separate
        // from the User Interface
        private void ScanThreadMain()
        {
            // we will use encoder sync mode here
            scanner.EnterEncoderSyncMode();
            // the code on a thread can not directly make calls on the 
            // UI thread, hence the need for Invoke()
            this.Invoke(new Action(() => this.textBox.Clear()));
            this.Invoke(new Action(() => this.textBox.AppendText("Scanning Started.\n")));
            
            // this is an endless loop until the isRunning variable is changed 
            // to false from the outside of this thread
            while (isRunning)
            {
                // GetQueuedProfiles() is a shortcut for BeginGetQueuedProfiles()/EndGetQueuedProfiles() 
                // See the Example on Hi-Speed Scanning for more information 
                List<Profile> received = scanner.GetQueuedProfiles(1);
                if (received.Count > 0)
                {
                    this.Invoke(new Action(() => 
                        this.textBox.AppendText(String.Format("Profile with {0} points measured.\n", 
                        received[0].Count))));
                }
            }

            scanner.ExitSyncMode();
            this.Invoke(new Action(() => this.textBox.AppendText("Scanning Stopped.\n")));
        }

        private void StartButton_Click(object sender, EventArgs e)
        {
            if (scanThread == null || !scanThread.IsAlive)
            {
                // we create a new thread here and assign the ScanThreadMain as the code path
                // to be executed when the thread is started
                scanThread = new Thread(new ThreadStart(this.ScanThreadMain))
                {                    
                    // this construct is an initializer list in C#
                    IsBackground = true,
                    Name = "ScanThreadMain",
                    Priority = ThreadPriority.Highest
                };
                isRunning = true;
                scanThread.Start();
            }
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            StopScanning();
        }

        private void StopScanning()
        {
            isRunning = false;
            if (scanThread != null)
            {
                // wait a moment for the thread to close down
                scanThread.Join(100);
                scanThread = null;
            }
        }

        private void QuitButton_Click(object sender, EventArgs e)
        {
            StopScanning();
            Close();
        }

        private Scanner SetupScanner()
        {
            // for clarity, no error checking here
            Scanner s = Scanner.Connect(new IPAddress(new byte[] {192, 168, 1, 150}));
            using (TextReader parametersReader = File.OpenText("param.dat"))
            {
                string parametersString = parametersReader.ReadToEnd();
                s.SetParameters(parametersString, true);
            }
            return s;
        }
    }
}
